@@ -250,9 +250,9 @@ impl<'ir> AsmBuilder<'ir> {
         let mut arg_word_offset = 0;
         for (name, val) in function.args_iter(self.context) {
             let current_arg_reg = self.value_to_register(val);
-            let arg_type_size_bytes =
-                ir_type_size_in_bytes(self.context, &val.get_type(self.context).unwrap());
-            if arg_type_size_bytes <= 8 {
+            let arg_type = val.get_type(self.context).unwrap();
+            let arg_type_size_bytes = ir_type_size_in_bytes(self.context, &arg_type);
+            if arg_type.is_copy_type() {
                 if arg_word_offset > crate::asm_generation::compiler_constants::TWELVE_BITS {
                     let offs_reg = self.reg_seqr.next();
                     self.bytecode.push(Op {
@@ -336,15 +336,15 @@ impl<'ir> AsmBuilder<'ir> {
                         stack_base += 1;
                     }
                     Type::B256 => {
+                        // XXX Like strings, should we just reserve space for a pointer?
                         self.ptr_map.insert(*ptr, Storage::Stack(stack_base));
                         stack_base += 4;
                     }
-                    Type::String(count) => {
+                    Type::String(_) => {
+                        // Strings are always constant and used by reference, so we only store the
+                        // pointer on the stack.
                         self.ptr_map.insert(*ptr, Storage::Stack(stack_base));
-
-                        // XXX `count` is a CHAR count, not BYTE count.  We need to count the size
-                        // of the string before allocating.  For now assuming CHAR == BYTE.
-                        stack_base += size_bytes_in_words!(count);
+                        stack_base += 1;
                     }
                     Type::Array(_) | Type::Struct(_) | Type::Union(_) => {
                         // Store this aggregate at the current stack base.
@@ -880,14 +880,14 @@ impl<'ir> AsmBuilder<'ir> {
     fn compile_extract_value(&mut self, instr_val: &Value, aggregate_val: &Value, indices: &[u64]) {
         // Base register should pointer to some stack allocated memory.
         let base_reg = self.value_to_register(aggregate_val);
-        let (extract_offset, value_size) = aggregate_idcs_to_field_layout(
+        let ((extract_offset, _), field_type) = aggregate_idcs_to_field_layout(
             self.context,
             &aggregate_val.get_type(self.context).unwrap(),
             indices,
         );
 
         let instr_reg = self.reg_seqr.next();
-        if value_size <= 8 {
+        if field_type.is_copy_type() {
             if extract_offset > crate::asm_generation::compiler_constants::TWELVE_BITS {
                 let offset_reg = self.reg_seqr.next();
                 self.number_to_reg(
@@ -1119,11 +1119,12 @@ impl<'ir> AsmBuilder<'ir> {
         value: &Value,
         indices: &[u64],
     ) {
+        println!("Compiling insert value");
         // Base register should point to some stack allocated memory.
         let base_reg = self.value_to_register(aggregate_val);
 
         let insert_reg = self.value_to_register(value);
-        let (insert_offs, value_size) = aggregate_idcs_to_field_layout(
+        let ((insert_offs, value_size), _) = aggregate_idcs_to_field_layout(
             self.context,
             &aggregate_val.get_type(self.context).unwrap(),
             indices,
@@ -1134,7 +1135,7 @@ impl<'ir> AsmBuilder<'ir> {
             .map(|idx| format!("{}", idx))
             .collect::<Vec<String>>()
             .join(",");
-        if value_size <= 8 {
+        if value.get_type(self.context).unwrap().is_copy_type() {
             if insert_offs > crate::asm_generation::compiler_constants::TWELVE_BITS {
                 let insert_offs_reg = self.reg_seqr.next();
                 self.number_to_reg(
@@ -1923,7 +1924,7 @@ impl<'ir> AsmBuilder<'ir> {
             ConstantValue::Bool(_) => 8,
             ConstantValue::Uint(_) => 8,
             ConstantValue::B256(_) => 32,
-            ConstantValue::String(s) => s.len() as u64, // String::len() returns the byte size, not char count.
+            ConstantValue::String(_) => 8,
             ConstantValue::Array(elems) => {
                 if elems.is_empty() {
                     0
@@ -1953,7 +1954,8 @@ impl<'ir> AsmBuilder<'ir> {
             ConstantValue::Unit
             | ConstantValue::Bool(_)
             | ConstantValue::Uint(_)
-            | ConstantValue::B256(_) => {
+            | ConstantValue::B256(_)
+            | ConstantValue::String(_) => {
                 // Get the constant into the namespace.
                 let lit = ir_constant_to_ast_literal(constant);
                 let data_id = self.data_section.insert_data_value(&lit);
@@ -2045,18 +2047,6 @@ impl<'ir> AsmBuilder<'ir> {
                 }
             }
 
-            ConstantValue::String(_) => {
-                // These are still not properly implemented until we refactor for spans!  There's
-                // an issue on GitHub for it.
-                self.bytecode.push(Op {
-                    opcode: Either::Left(VirtualOp::NOOP),
-                    comment: "strings aren't implemented!".into(),
-                    owning_span: span,
-                });
-
-                0
-            }
-
             ConstantValue::Array(items) | ConstantValue::Struct(items) => {
                 // Recurse for each item, accumulating the field offset and the final size.
                 items.iter().fold(0, |local_offs, item| {
@@ -2091,10 +2081,14 @@ fn ir_constant_to_ast_literal(constant: &Constant) -> Literal {
         ConstantValue::Bool(b) => Literal::Boolean(*b),
         ConstantValue::Uint(n) => Literal::U64(*n),
         ConstantValue::B256(bs) => Literal::B256(*bs),
-        ConstantValue::String(str) => Literal::String(crate::span::Span {
-            span: pest::Span::new(std::sync::Arc::from(str.as_str()), 0, str.len()).unwrap(),
-            path: None,
-        }),
+        ConstantValue::String(bs) => {
+            // ConstantValue::String bytes are guaranteed to be valid UTF8.
+            let s = std::str::from_utf8(bs).unwrap();
+            Literal::String(crate::span::Span {
+                span: pest::Span::new(std::sync::Arc::from(s), 0, s.len()).unwrap(),
+                path: None,
+            })
+        }
         ConstantValue::Array(_) | ConstantValue::Struct(_) => {
             unreachable!("Cannot convert aggregates to a literal.")
         }
@@ -2107,7 +2101,7 @@ pub fn ir_type_size_in_bytes(context: &Context, ty: &Type) -> u64 {
     match ty {
         Type::Unit | Type::Bool | Type::Uint(_) => 8,
         Type::B256 => 32,
-        Type::String(n) => *n,
+        Type::String(_) => 8,
         Type::Array(aggregate) => {
             if let AggregateContent::ArrayType(el_ty, cnt) = &context.aggregates[aggregate.0] {
                 cnt * ir_type_size_in_bytes(context, el_ty)
@@ -2142,7 +2136,11 @@ pub fn ir_type_size_in_bytes(context: &Context, ty: &Type) -> u64 {
 }
 
 // Aggregate (nested) field offset in words and size in bytes.
-pub fn aggregate_idcs_to_field_layout(context: &Context, ty: &Type, idcs: &[u64]) -> (u64, u64) {
+pub fn aggregate_idcs_to_field_layout(
+    context: &Context,
+    ty: &Type,
+    idcs: &[u64],
+) -> ((u64, u64), Type) {
     idcs.iter()
         .fold(((0, 0), *ty), |((offs, _), ty), idx| match ty {
             Type::Struct(aggregate) => {
@@ -2176,7 +2174,6 @@ pub fn aggregate_idcs_to_field_layout(context: &Context, ty: &Type, idcs: &[u64]
 
             _otherwise => panic!("Attempt to access field in non-aggregate."),
         })
-        .0
 }
 
 // -------------------------------------------------------------------------------------------------
@@ -2228,7 +2225,7 @@ mod tests {
                 file_name: std::sync::Arc::new("".into()),
                 dir_of_code: std::sync::Arc::new("".into()),
                 manifest_path: std::sync::Arc::new("".into()),
-                use_ir: false,
+                use_orig_asm: false,
                 print_intermediate_asm: false,
                 print_finalized_asm: false,
                 print_ir: false,
