@@ -715,11 +715,7 @@ impl NamespaceWrapper for NamespaceRef {
                         }
                         if !decl.type_parameters.is_empty() {
                             let new_decl = check!(
-                                decl.monomorphize_with_type_arguments(
-                                    self,
-                                    &new_type_arguments,
-                                    Some(self_type)
-                                ),
+                                decl.monomorphize(self, &new_type_arguments, Some(self_type)),
                                 return err(warnings, errors),
                                 warnings,
                                 errors
@@ -801,7 +797,7 @@ impl NamespaceWrapper for NamespaceRef {
                     }
                     if !decl.type_parameters.is_empty() {
                         let new_decl = check!(
-                            decl.monomorphize_with_type_arguments(self, &new_type_arguments, None),
+                            decl.monomorphize(self, &new_type_arguments, None),
                             return err(warnings, errors),
                             warnings,
                             errors
