@@ -62,35 +62,37 @@ fn generic_enum_resolution() {
     use crate::Ident;
     let engine = Engine::default();
 
-    let sp = Span {
-        span: pest::Span::new(" ".into(), 0, 0).unwrap(),
-        path: None,
-    };
+    let sp = Span::empty();
+
+    let variant_types = vec![TypedEnumVariant {
+        name: Ident::new_with_override("a", sp.clone()),
+        tag: 0,
+        r#type: engine.insert_type(TypeInfo::UnknownGeneric {
+            name: Ident::new_with_override("T", sp.clone()),
+        }),
+        span: sp.clone(),
+    }];
 
     let ty_1 = engine.insert_type(TypeInfo::Enum {
         name: Ident::new_with_override("Result", sp.clone()),
-        variant_types: vec![TypedEnumVariant {
-            name: Ident::new_with_override("a", sp.clone()),
-            tag: 0,
-            r#type: engine.insert_type(TypeInfo::UnknownGeneric {
-                name: Ident::new_with_override("T", sp.clone()),
-            }),
-            span: sp.clone(),
-        }],
+        variant_types,
     });
 
+    let variant_types = vec![TypedEnumVariant {
+        name: Ident::new_with_override("a", sp.clone()),
+        tag: 0,
+        r#type: engine.insert_type(TypeInfo::Boolean),
+        span: sp.clone(),
+    }];
+
     let ty_2 = engine.insert_type(TypeInfo::Enum {
         name: Ident::new_with_override("Result", sp.clone()),
-        variant_types: vec![TypedEnumVariant {
-            name: Ident::new_with_override("a", sp.clone()),
-            tag: 0,
-            r#type: engine.insert_type(TypeInfo::Boolean),
-            span: sp.clone(),
-        }],
+        variant_types,
     });
 
     // Unify them together...
-    engine.unify(ty_1, ty_2, &sp, "").unwrap();
+    let (_, errors) = engine.unify(ty_1, ty_2, &sp, "");
+    assert!(errors.is_empty());
 
     if let TypeInfo::Enum {
         name,
@@ -111,16 +113,14 @@ fn generic_enum_resolution() {
 fn basic_numeric_unknown() {
     let engine = Engine::default();
 
-    let sp = Span {
-        span: pest::Span::new(" ".into(), 0, 0).unwrap(),
-        path: None,
-    };
+    let sp = Span::empty();
     // numerics
     let id = engine.insert_type(TypeInfo::Numeric);
     let id2 = engine.insert_type(TypeInfo::UnsignedInteger(IntegerBits::Eight));
 
     // Unify them together...
-    engine.unify(id, id2, &sp, "").unwrap();
+    let (_, errors) = engine.unify(id, id2, &sp, "");
+    assert!(errors.is_empty());
 
     assert_eq!(
         engine.resolve_type(id, &sp).unwrap(),
@@ -130,10 +130,7 @@ fn basic_numeric_unknown() {
 #[test]
 fn chain_of_refs() {
     let engine = Engine::default();
-    let sp = Span {
-        span: pest::Span::new(" ".into(), 0, 0).unwrap(),
-        path: None,
-    };
+    let sp = Span::empty();
     // numerics
     let id = engine.insert_type(TypeInfo::Numeric);
     let id2 = engine.insert_type(TypeInfo::Ref(id));
@@ -141,7 +138,8 @@ fn chain_of_refs() {
     let id4 = engine.insert_type(TypeInfo::UnsignedInteger(IntegerBits::Eight));
 
     // Unify them together...
-    engine.unify(id4, id2, &sp, "").unwrap();
+    let (_, errors) = engine.unify(id4, id2, &sp, "");
+    assert!(errors.is_empty());
 
     assert_eq!(
         engine.resolve_type(id3, &sp).unwrap(),
@@ -151,10 +149,7 @@ fn chain_of_refs() {
 #[test]
 fn chain_of_refs_2() {
     let engine = Engine::default();
-    let sp = Span {
-        span: pest::Span::new(" ".into(), 0, 0).unwrap(),
-        path: None,
-    };
+    let sp = Span::empty();
     // numerics
     let id = engine.insert_type(TypeInfo::Numeric);
     let id2 = engine.insert_type(TypeInfo::Ref(id));
@@ -162,7 +157,8 @@ fn chain_of_refs_2() {
     let id4 = engine.insert_type(TypeInfo::UnsignedInteger(IntegerBits::Eight));
 
     // Unify them together...
-    engine.unify(id2, id4, &sp, "").unwrap();
+    let (_, errors) = engine.unify(id2, id4, &sp, "");
+    assert!(errors.is_empty());
 
     assert_eq!(
         engine.resolve_type(id3, &sp).unwrap(),
@@ -191,100 +187,36 @@ fn parse_str_type(raw: &str, span: Span) -> CompileResult<TypeInfo> {
 
 #[test]
 fn test_str_parse() {
-    match parse_str_type(
-        "str[20]",
-        Span {
-            span: pest::Span::new("".into(), 0, 0).unwrap(),
-            path: None,
-        },
-    )
-    .value
-    {
+    match parse_str_type("str[20]", Span::empty()).value {
         Some(value) if value == TypeInfo::Str(20) => (),
         _ => panic!("failed test"),
     }
-    match parse_str_type(
-        "str[]",
-        Span {
-            span: pest::Span::new("".into(), 0, 0).unwrap(),
-            path: None,
-        },
-    )
-    .value
-    {
+    match parse_str_type("str[]", Span::empty()).value {
         None => (),
         _ => panic!("failed test"),
     }
-    match parse_str_type(
-        "str[ab]",
-        Span {
-            span: pest::Span::new("".into(), 0, 0).unwrap(),
-            path: None,
-        },
-    )
-    .value
-    {
+    match parse_str_type("str[ab]", Span::empty()).value {
         None => (),
         _ => panic!("failed test"),
     }
-    match parse_str_type(
-        "str [ab]",
-        Span {
-            span: pest::Span::new("".into(), 0, 0).unwrap(),
-            path: None,
-        },
-    )
-    .value
-    {
+    match parse_str_type("str [ab]", Span::empty()).value {
         None => (),
         _ => panic!("failed test"),
     }
 
-    match parse_str_type(
-        "not even a str[ type",
-        Span {
-            span: pest::Span::new("".into(), 0, 0).unwrap(),
-            path: None,
-        },
-    )
-    .value
-    {
+    match parse_str_type("not even a str[ type", Span::empty()).value {
         None => (),
         _ => panic!("failed test"),
     }
-    match parse_str_type(
-        "",
-        Span {
-            span: pest::Span::new("".into(), 0, 0).unwrap(),
-            path: None,
-        },
-    )
-    .value
-    {
+    match parse_str_type("", Span::empty()).value {
         None => (),
         _ => panic!("failed test"),
     }
-    match parse_str_type(
-        "20",
-        Span {
-            span: pest::Span::new("".into(), 0, 0).unwrap(),
-            path: None,
-        },
-    )
-    .value
-    {
+    match parse_str_type("20", Span::empty()).value {
         None => (),
         _ => panic!("failed test"),
     }
-    match parse_str_type(
-        "[20]",
-        Span {
-            span: pest::Span::new("".into(), 0, 0).unwrap(),
-            path: None,
-        },
-    )
-    .value
-    {
+    match parse_str_type("[20]", Span::empty()).value {
         None => (),
         _ => panic!("failed test"),
     }
